﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using MVR;
using MVR.FileManagementSecure;
using MeshVR;
using static ClockwiseSilver.Helpers;

namespace ClockwiseSilver
{
	public class Circler : AnimatedElement
	{
		private JSONStorableFloat radiusMinJSON, radiusMaxJSON, speedXMinJSON, speedXMaxJSON,
			speedYMinJSON, speedYMaxJSON, timeMinJSON, timeMaxJSON;
		
		private Vector3 currentRot = Vector3.zero;
		private float targetRadius, lastRadius, targetSpeedX, lastSpeedX, targetSpeedY, lastSpeedY, timer,
			timeScale, progress = 0f;
		
		public Circler(string targetID, Atom atom)
		{
			this.targetID = targetID;
			this.atom = atom;
			name = "Circler_" + targetID;
			
			if (targetID != null && targetID != "" && targetID != "None")
			{
				body = atom.rigidbodies.First(rb => rb.name == targetID);
				controller = atom.freeControllers.First(fc => fc.name == Helpers.RigidbodyToFreeController(targetID));
				controllerTransform = controller.transform;
			}
		}
		
		public override void Init(MVRScript script)
		{
			base.Init(script);
			
			normalColor = new Color(0.2f,0.0f,0.4f);
			activeColor = new Color(0.9f,0.9f,0.4f);
			
			radiusMinJSON = new JSONStorableFloat(targetID + " Radius Min", 50f, 1f, 400f, true);
			script.RegisterFloat(radiusMinJSON);
			sliders.Add(radiusMinJSON);
			radiusMaxJSON = new JSONStorableFloat(targetID + " Radius Max", 50f, 1f, 400f, true);
			script.RegisterFloat(radiusMaxJSON);
			sliders.Add(radiusMaxJSON);
			
			speedXMinJSON = new JSONStorableFloat(targetID + " Speed X Min", 0f, -500f, 500f, true);
			script.RegisterFloat(speedXMinJSON);
			sliders.Add(speedXMinJSON);
			speedXMaxJSON = new JSONStorableFloat(targetID + " Speed X Max", 0f, -500f, 500f, true);
			script.RegisterFloat(speedXMaxJSON);
			sliders.Add(speedXMaxJSON);
			
			speedYMinJSON = new JSONStorableFloat(targetID + " Speed Y Min", 0f, -500f, 500f, true);
			script.RegisterFloat(speedYMinJSON);
			sliders.Add(speedYMinJSON);
			speedYMaxJSON = new JSONStorableFloat(targetID + " Speed Y Max", 0f, -500f, 500f, true);
			script.RegisterFloat(speedYMaxJSON);
			sliders.Add(speedYMaxJSON);
			
			timeMinJSON = new JSONStorableFloat(targetID + "C Random Time Min", 4f, 1f, 30f, true);
			script.RegisterFloat(timeMinJSON);
			sliders.Add(timeMinJSON);
			timeMaxJSON = new JSONStorableFloat(targetID + "C Random Time Max", 12f, 1f, 30f, true);
			script.RegisterFloat(timeMaxJSON);
			sliders.Add(timeMaxJSON);
		}
		
		public override void Start(MVRScript script)
		{
			base.Start(script);
			if (isActive.val) { SetActive(true); }
		}
		
		public override void RestoreFromJson(JSONNode node)
        {
			base.RestoreFromJson(node);
			if (isActive.val) { SetActive(true); }
		}
		
		public override void CreateMenu(MVRScript script)
		{
			base.CreateMenu(script);
			
			script.CreateSlider(radiusMinJSON, true);
			script.CreateSlider(radiusMaxJSON, true);
			script.CreateSlider(speedXMinJSON, true);
			script.CreateSlider(speedXMaxJSON, true);
			script.CreateSlider(speedYMinJSON, true);
			script.CreateSlider(speedYMaxJSON, true);
			script.CreateSlider(timeMinJSON, true);
			script.CreateSlider(timeMaxJSON, true);
			
			isActive.toggle.onValueChanged.AddListener(checkVal =>
			{
				SetActive(checkVal);
			});
		}
		
		public override void Update(float dt, float intensity)
		{
			if (!active) { return; }
			base.Update(dt, intensity);

			progress = Mathf.Clamp01(progress + (dt * timeScale));
			float eased = Helpers.QuartOut(progress);
			currentRot.x = Helpers.Rollover(currentRot.x + (Mathf.Lerp(lastSpeedX, targetSpeedX, eased) * dt));
			currentRot.y = Helpers.Rollover(currentRot.y + (Mathf.Lerp(lastSpeedY, targetSpeedY, eased) * dt));			
			Vector3 dir = controllerTransform.TransformDirection(Quaternion.Euler(currentRot) * Vector3.forward);			
			body.AddForce(dir * Mathf.Lerp(lastRadius, targetRadius, eased) * intensity, ForceMode.Force);
			
			timer -= dt;
			if (timer <= 0f) { SetTargets(); }
		}
		
		private void StartCircler()
		{
			 active = true;
		}
		
		public override void SetActive(bool setActive)
        {
			base.SetActive(setActive);
			if (setActive) { currentRot = Vector3.zero; SetTargets(); SetTargets(); }
		}
		
		private void SetTargets()
		{
			if (radiusMinJSON.val > radiusMaxJSON.val) { radiusMaxJSON.val = radiusMinJSON.val + 5f; }
			if (speedXMinJSON.val > speedXMaxJSON.val) { speedXMaxJSON.val = speedXMinJSON.val + 5f; }
			if (speedYMinJSON.val > speedYMaxJSON.val) { speedYMaxJSON.val = speedYMinJSON.val + 5f; }
			if (timeMinJSON.val > timeMaxJSON.val) { timeMaxJSON.val = timeMinJSON.val + 5f; }
			lastRadius = targetRadius;
			lastSpeedX = targetSpeedX;
			lastSpeedY = targetSpeedY;
			targetRadius = UnityEngine.Random.Range(radiusMinJSON.val, radiusMaxJSON.val);
			targetSpeedX = UnityEngine.Random.Range(speedXMinJSON.val, speedXMaxJSON.val);
			targetSpeedY = UnityEngine.Random.Range(speedYMinJSON.val, speedYMaxJSON.val);
			timer = UnityEngine.Random.Range(timeMinJSON.val, timeMaxJSON.val) * timeMultiplierJSON.val;
			if (timer <= 0.1f) { timer = 0.1f; }
			timeScale = 1f / timer;
			progress = 0f;
		}
	}
}