﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using MVR;
using MVR.FileManagementSecure;
using MeshVR;
using static ClockwiseSilver.Helpers;

namespace ClockwiseSilver
{
	public class JointDrive : AnimatedElement
	{
		private JSONStorableFloat springMinJSON, springMaxJSON, xMinJSON, xMaxJSON, yMinJSON, yMaxJSON,
			zMinJSON, zMaxJSON, forceMinJSON, forceMaxJSON, timeMinJSON, timeMaxJSON,
			springDefaultJSON, xDefaultJSON, yDefaultJSON, zDefaultJSON, forceDefaultJSON;
		
		private Vector3 currentDrive, lastDrive, targetDrive = Vector3.zero;
		private float targetSpring, lastSpring, targetForce, lastForce,
			timer, timeScale, progress = 0f;
		
		public JointDrive(string targetID, Atom atom)
		{
			this.targetID = targetID;
			this.atom = atom;
			name = "JointDrive_" + targetID;
			
			if (targetID != null && targetID != "" && targetID != "None")
			{
				body = atom.rigidbodies.First(rb => rb.name == targetID);
				controller = atom.freeControllers.First(fc => fc.name == Helpers.RigidbodyToFreeController(targetID));
				controllerTransform = controller.transform;
			}
		}
		
		public override void Init(MVRScript script)
		{
			base.Init(script);
			
			normalColor = new Color(0.3f,0.0f,0.4f);
			activeColor = new Color(0.9f,0.9f,0.4f);
			
			springMinJSON = new JSONStorableFloat(targetID + " Spring Min", 0f, 0f, 140f, true);
			script.RegisterFloat(springMinJSON);
			sliders.Add(springMinJSON);
			springMaxJSON = new JSONStorableFloat(targetID + " Spring Max", 0f, 0f, 140f, true);
			script.RegisterFloat(springMaxJSON);
			sliders.Add(springMaxJSON);
			
			xMinJSON = new JSONStorableFloat(targetID + " Joint Drive X Min", 0f, -140f, 140f, true);
			script.RegisterFloat(xMinJSON);
			sliders.Add(xMinJSON);
			xMaxJSON = new JSONStorableFloat(targetID + " Joint Drive X Max", 0f, -140f, 140f, true);
			script.RegisterFloat(xMaxJSON);
			sliders.Add(xMaxJSON);
			
			yMinJSON = new JSONStorableFloat(targetID + " Joint Drive Y Min", 0f, -140f, 140f, true);
			script.RegisterFloat(yMinJSON);
			sliders.Add(yMinJSON);
			yMaxJSON = new JSONStorableFloat(targetID + " Joint Drive Y Max", 0f, -140f, 140f, true);
			script.RegisterFloat(yMaxJSON);
			sliders.Add(yMaxJSON);
			
			zMinJSON = new JSONStorableFloat(targetID + " Joint Drive Z Min", 0f, -140f, 140f, true);
			script.RegisterFloat(zMinJSON);
			sliders.Add(zMinJSON);
			zMaxJSON = new JSONStorableFloat(targetID + " Joint Drive Z Max", 0f, -140f, 140f, true);
			script.RegisterFloat(zMaxJSON);
			sliders.Add(zMaxJSON);
			
			forceMinJSON = new JSONStorableFloat(targetID + " Max Force Min", 50f, 0f, 200f, true);
			script.RegisterFloat(forceMinJSON);
			sliders.Add(forceMinJSON);
			forceMaxJSON = new JSONStorableFloat(targetID + " Max Force Max", 150f, 0f, 200f, true);
			script.RegisterFloat(forceMaxJSON);
			sliders.Add(forceMaxJSON);
			
			timeMinJSON = new JSONStorableFloat(targetID + "J Random Time Min", 0.5f, 0.1f, 10f, true);
			script.RegisterFloat(timeMinJSON);
			sliders.Add(timeMinJSON);
			timeMaxJSON = new JSONStorableFloat(targetID + "J Random Time Max", 4f, 0.1f, 10f, true);
			script.RegisterFloat(timeMaxJSON);
			sliders.Add(timeMaxJSON);
			
			springDefaultJSON = new JSONStorableFloat(targetID + "JSpringDefault", -200f, -200f, 200f, true);
			script.RegisterFloat(springDefaultJSON);
			xDefaultJSON = new JSONStorableFloat(targetID + "JXDefault", -200f, -200f, 200f, true);
			script.RegisterFloat(xDefaultJSON);
			yDefaultJSON = new JSONStorableFloat(targetID + "JYDefault", -200f, -200f, 200f, true);
			script.RegisterFloat(yDefaultJSON);
			zDefaultJSON = new JSONStorableFloat(targetID + "JZDefault", -200f, -200f, 200f, true);
			script.RegisterFloat(zDefaultJSON);
			forceDefaultJSON = new JSONStorableFloat(targetID + "JForceDefault", -200f, -200f, 200f, true);
			script.RegisterFloat(forceDefaultJSON);
		}
		
		public override void Start(MVRScript script)
		{
			base.Start(script);
			SetDefaults();
			if (isActive.val) { SetActive(true); }
		}
		
		public override void RestoreFromJson(JSONNode node)
        {
			base.RestoreFromJson(node);
			if (isActive.val) { SetActive(true); }
		}
		
		public override void CreateMenu(MVRScript script)
		{
			base.CreateMenu(script);			
			script.CreateSlider(springMinJSON, true);
			script.CreateSlider(springMaxJSON, true);
			script.CreateSlider(xMinJSON, true);
			script.CreateSlider(xMaxJSON, true);
			script.CreateSlider(yMinJSON, true);
			script.CreateSlider(yMaxJSON, true);
			script.CreateSlider(zMinJSON, true);
			script.CreateSlider(zMaxJSON, true);
			script.CreateSlider(forceMinJSON, true);
			script.CreateSlider(forceMaxJSON, true);
			script.CreateSlider(timeMinJSON, true);
			script.CreateSlider(timeMaxJSON, true);
			
			isActive.toggle.onValueChanged.AddListener(checkVal =>
			{
				SetActive(checkVal);
			});
			
			springMinJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveSpring = value; } });
			springMaxJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveSpring = value; } });
			xMinJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveXTarget = value; } });
			xMaxJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveXTarget = value; } });
			yMinJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveYTarget = value; } });
			yMaxJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveYTarget = value; } });
			zMinJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveZTarget = value; } });
			zMaxJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveZTarget = value; } });
			forceMinJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveMaxForce = value; } });
			forceMaxJSON.slider.onValueChanged.AddListener((value) => { if (!active) { controller.jointRotationDriveMaxForce = value; } });
		}
		
		public override void Update(float dt, float intensity)
		{
			if (!active) { return; }
			base.Update(dt, intensity);		
			progress = Mathf.Clamp01(progress + (dt * timeScale));
			float eased = Helpers.QuartOut(progress);				
			currentDrive = Vector3.Lerp(lastDrive, targetDrive, eased);
			controller.jointRotationDriveXTarget = currentDrive.x * intensity;
			controller.jointRotationDriveYTarget = currentDrive.y * intensity;
			controller.jointRotationDriveZTarget = currentDrive.z * intensity;
			controller.jointRotationDriveSpring = Mathf.Lerp(lastSpring, targetSpring, eased) * intensity;
			controller.jointRotationDriveMaxForce = Mathf.Lerp(lastForce, targetForce, eased);			
			timer -= dt;
			if (timer <= 0f) { SetTargets(); }
		}
		
		public override void OnDisable(MVRScript script)
		{
			base.OnDisable(script);
			RestoreDefaults();
		}
		
		public override void SetActive(bool setActive)
        {
			base.SetActive(setActive);
			if (setActive) { SetTargets(); SetTargets(); }
			else { RestoreDefaults(); }
		}
		
		private void SetDefaults(bool force=false)
		{
			if (force || springDefaultJSON.val == -200f)
			{
				springDefaultJSON.val = controller.jointRotationDriveSpring;
				springMinJSON.val = springMaxJSON.val = controller.jointRotationDriveSpring;
			}
			if (force || xDefaultJSON.val == -200f)
			{
				xDefaultJSON.val = controller.jointRotationDriveXTarget;
				xMinJSON.val = xMaxJSON.val = controller.jointRotationDriveXTarget;
			}
			if (force || yDefaultJSON.val == -200f)
			{
				yDefaultJSON.val = controller.jointRotationDriveYTarget;
				yMinJSON.val = yMaxJSON.val = controller.jointRotationDriveYTarget;
			}
			if (force || zDefaultJSON.val == -200f)
			{
				zDefaultJSON.val = controller.jointRotationDriveZTarget;
				zMinJSON.val = zMaxJSON.val = controller.jointRotationDriveZTarget;
			}
			if (force || forceDefaultJSON.val == -200f)
			{
				forceDefaultJSON.val = controller.jointRotationDriveMaxForce;
				forceMinJSON.val = forceMaxJSON.val = controller.jointRotationDriveMaxForce;
			}
			
			springMinJSON.defaultVal = springDefaultJSON.val;
			springMaxJSON.defaultVal = springDefaultJSON.val;
			xMinJSON.defaultVal = xDefaultJSON.val;
			xMaxJSON.defaultVal = xDefaultJSON.val;
			yMinJSON.defaultVal = yDefaultJSON.val;
			yMaxJSON.defaultVal = yDefaultJSON.val;
			zMinJSON.defaultVal = zDefaultJSON.val;
			zMaxJSON.defaultVal = zDefaultJSON.val;
			forceMinJSON.defaultVal = forceDefaultJSON.val;
			forceMaxJSON.defaultVal = forceDefaultJSON.val;
		}
		
		private void RestoreDefaults()
		{
			controller.jointRotationDriveSpring = springDefaultJSON.val;
			controller.jointRotationDriveXTarget = xDefaultJSON.val;
			controller.jointRotationDriveYTarget = yDefaultJSON.val;
			controller.jointRotationDriveZTarget = zDefaultJSON.val;
			controller.jointRotationDriveMaxForce = forceDefaultJSON.val;
		}
		
		private void SetTargets()
		{
			if (springMinJSON.val > springMaxJSON.val) { springMaxJSON.val = springMinJSON.val + 5f; }
			if (xMinJSON.val > xMaxJSON.val) { xMaxJSON.val = xMinJSON.val + 5f; }
			if (yMinJSON.val > yMaxJSON.val) { yMaxJSON.val = yMinJSON.val + 5f; }
			if (zMinJSON.val > zMaxJSON.val) { zMaxJSON.val = zMinJSON.val + 5f; }
			if (forceMinJSON.val > forceMaxJSON.val) { forceMaxJSON.val = forceMinJSON.val + 5f; }
			if (timeMinJSON.val > timeMaxJSON.val) { timeMaxJSON.val = timeMinJSON.val + 5f; }
			lastSpring = targetSpring;
			lastForce = targetForce;
			lastDrive = targetDrive;
			targetSpring = UnityEngine.Random.Range(springMinJSON.val, springMaxJSON.val);
			targetDrive.x = UnityEngine.Random.Range(xMinJSON.val, xMaxJSON.val);
			targetDrive.y = UnityEngine.Random.Range(yMinJSON.val, yMaxJSON.val);
			targetDrive.z = UnityEngine.Random.Range(zMinJSON.val, zMaxJSON.val);
			targetForce = UnityEngine.Random.Range(forceMinJSON.val, forceMaxJSON.val);
			timer = UnityEngine.Random.Range(timeMinJSON.val, timeMaxJSON.val) * timeMultiplierJSON.val;
			if (timer <= 0.1f) { timer = 0.1f; }
			timeScale = 1f / timer;
			progress = 0f;
		}
	}
}