﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using MVR;
using MVR.FileManagementSecure;
using MeshVR;
using static ClockwiseSilver.Helpers;

namespace ClockwiseSilver
{
	public class RigidbodyForce : AnimatedElement
	{
		private JSONStorableFloat xPosMinJSON, xPosMaxJSON, yPosMinJSON, yPosMaxJSON, zPosMinJSON, zPosMaxJSON,
			xRotMinJSON, xRotMaxJSON, yRotMinJSON, yRotMaxJSON, zRotMinJSON, zRotMaxJSON,
			timeMinJSON, timeMaxJSON;
		
		private Vector3 currentPos, lastPos, targetPos, currentRot, lastRot, targetRot = Vector3.zero;
		private float timer, timeScale, progress = 0f;
		//private Transform referenceTransform;
		
		public RigidbodyForce(string targetID, Atom atom)
		{
			this.targetID = targetID;
			this.atom = atom;
			name = "RigidbodyForce_" + targetID;
			
			if (targetID != null && targetID != "" && targetID != "None")
			{
				body = atom.rigidbodies.First(rb => rb.name == targetID);
				controller = atom.freeControllers.First(fc => fc.name == Helpers.RigidbodyToFreeController(targetID));
				controllerTransform = controller.transform;
			}
		}
		
		public override void Init(MVRScript script)
		{
			base.Init(script);
			
			normalColor = new Color(0.4f,0.0f,0.2f);
			activeColor = new Color(0.9f,0.9f,0.4f);
			
			xPosMinJSON = new JSONStorableFloat(targetID + " X Pos Min", 0f, -5000f, 5000f, true);
			script.RegisterFloat(xPosMinJSON);
			sliders.Add(xPosMinJSON);
			xPosMaxJSON = new JSONStorableFloat(targetID + " X Pos Max", 0f, -5000f, 5000f, true);
			script.RegisterFloat(xPosMaxJSON);
			sliders.Add(xPosMaxJSON);
			
			yPosMinJSON = new JSONStorableFloat(targetID + " Y Pos Min", 0f, -5000f, 5000f, true);
			script.RegisterFloat(yPosMinJSON);
			sliders.Add(yPosMinJSON);
			yPosMaxJSON = new JSONStorableFloat(targetID + " Y Pos Max", 0f, -5000f, 5000f, true);
			script.RegisterFloat(yPosMaxJSON);
			sliders.Add(yPosMaxJSON);
			
			zPosMinJSON = new JSONStorableFloat(targetID + " Z Pos Min", 0f, -5000f, 5000f, true);
			script.RegisterFloat(zPosMinJSON);
			sliders.Add(zPosMinJSON);
			zPosMaxJSON = new JSONStorableFloat(targetID + " Z Pos Max", 0f, -5000f, 5000f, true);
			script.RegisterFloat(zPosMaxJSON);
			sliders.Add(zPosMaxJSON);
			
			xRotMinJSON = new JSONStorableFloat(targetID + " X Rot Min", 0f, -2500f, 2500f, true);
			script.RegisterFloat(xRotMinJSON);
			sliders.Add(xRotMinJSON);
			xRotMaxJSON = new JSONStorableFloat(targetID + " X Rot Max", 0f, -2500f, 2500f, true);
			script.RegisterFloat(xRotMaxJSON);
			sliders.Add(xRotMaxJSON);
			
			yRotMinJSON = new JSONStorableFloat(targetID + " Y Rot Min", 0f, -2500f, 2500f, true);
			script.RegisterFloat(yRotMinJSON);
			sliders.Add(yRotMinJSON);
			yRotMaxJSON = new JSONStorableFloat(targetID + " Y Rot Max", 0f, -2500f, 2500f, true);
			script.RegisterFloat(yRotMaxJSON);
			sliders.Add(yRotMaxJSON);
			
			zRotMinJSON = new JSONStorableFloat(targetID + " Z Rot Min", 0f, -2500f, 2500f, true);
			script.RegisterFloat(zRotMinJSON);
			sliders.Add(zRotMinJSON);
			zRotMaxJSON = new JSONStorableFloat(targetID + " Z Rot Max", 0f, -2500f, 2500f, true);
			script.RegisterFloat(zRotMaxJSON);
			sliders.Add(zRotMaxJSON);
			
			timeMinJSON = new JSONStorableFloat(targetID + "R Random Time Min", 0.5f, 0.1f, 10f, true);
			script.RegisterFloat(timeMinJSON);
			sliders.Add(timeMinJSON);
			timeMaxJSON = new JSONStorableFloat(targetID + "R Random Time Max", 4f, 0.1f, 10f, true);
			script.RegisterFloat(timeMaxJSON);
			sliders.Add(timeMaxJSON);
		}
		
		public override void Start(MVRScript script)
		{
			base.Start(script);
			if (isActive.val) { SetActive(true); }
		}
		
		public override void RestoreFromJson(JSONNode node)
        {
			base.RestoreFromJson(node);
			if (isActive.val) { SetActive(true); }
		}
		
		public override void CreateMenu(MVRScript script)
		{
			base.CreateMenu(script);
			script.CreateSlider(xPosMinJSON, true);
			script.CreateSlider(xPosMaxJSON, true);
			script.CreateSlider(yPosMinJSON, true);
			script.CreateSlider(yPosMaxJSON, true);
			script.CreateSlider(zPosMinJSON, true);
			script.CreateSlider(zPosMaxJSON, true);
			script.CreateSlider(xRotMinJSON, true);
			script.CreateSlider(xRotMaxJSON, true);
			script.CreateSlider(yRotMinJSON, true);
			script.CreateSlider(yRotMaxJSON, true);
			script.CreateSlider(zRotMinJSON, true);
			script.CreateSlider(zRotMaxJSON, true);
			script.CreateSlider(timeMinJSON, true);
			script.CreateSlider(timeMaxJSON, true);
			
			isActive.toggle.onValueChanged.AddListener(checkVal =>
			{
				SetActive(checkVal);
			});
		}
		
		public override void Update(float dt, float intensity)
		{
			if (!active) { return; }
			base.Update(dt, intensity);		
			progress = Mathf.Clamp01(progress + (dt * timeScale));
			float eased = Helpers.QuadraticInOut(progress);			
			
			currentPos = Vector3.Lerp(lastPos, targetPos, eased);
			currentRot = Vector3.Lerp(lastRot, targetRot, eased);
			dt *= intensity;
			body.AddRelativeForce(currentPos * (dt * body.mass), ForceMode.Force);
			body.AddRelativeTorque(currentRot * dt, ForceMode.Force);
			
			timer -= dt;
			if (timer <= 0f) { SetTargets(); }
		}
		
		public override void OnDisable(MVRScript script)
		{
			base.OnDisable(script);
		}
		
		public override void SetActive(bool setActive)
        {
			base.SetActive(setActive);
			if (setActive) { SetTargets(); SetTargets(); }
		}
		
		private void SetTargets()
		{
			if (xPosMinJSON.val > xPosMaxJSON.val) { xPosMaxJSON.val = xPosMinJSON.val + 5f; }
			if (yPosMinJSON.val > yPosMaxJSON.val) { yPosMaxJSON.val = yPosMinJSON.val + 5f; }
			if (zPosMinJSON.val > zPosMaxJSON.val) { zPosMaxJSON.val = zPosMinJSON.val + 5f; }
			if (xRotMinJSON.val > xRotMaxJSON.val) { xRotMaxJSON.val = xRotMinJSON.val + 5f; }
			if (yRotMinJSON.val > yRotMaxJSON.val) { yRotMaxJSON.val = yRotMinJSON.val + 5f; }
			if (zRotMinJSON.val > zRotMaxJSON.val) { zRotMaxJSON.val = zRotMinJSON.val + 5f; }
			
			lastPos = targetPos;
			lastRot = targetRot;
			
			targetPos.x = UnityEngine.Random.Range(xPosMinJSON.val, xPosMaxJSON.val);
			targetPos.y = UnityEngine.Random.Range(yPosMinJSON.val, yPosMaxJSON.val);
			targetPos.z = UnityEngine.Random.Range(zPosMinJSON.val, zPosMaxJSON.val);
			targetRot.x = UnityEngine.Random.Range(xRotMinJSON.val, xRotMaxJSON.val);
			targetRot.y = UnityEngine.Random.Range(yRotMinJSON.val, yRotMaxJSON.val);
			targetRot.z = UnityEngine.Random.Range(zRotMinJSON.val, zRotMaxJSON.val);
			
			//targetPos = controllerTransform.InverseTransformPoint(targetPos);
			//targetRot = controllerTransform.InverseTransformDirection(targetPos);
			
			timer = UnityEngine.Random.Range(timeMinJSON.val, timeMaxJSON.val) * timeMultiplierJSON.val;
			if (timer <= 0.1f) { timer = 0.1f; }
			timeScale = 1f / timer;
			progress = 0f;
		}
	}
}