﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using MVR;
using MVR.FileManagementSecure;
using MeshVR;
using static ClockwiseSilver.Helpers;

namespace ClockwiseSilver
{
	public abstract class AnimatedElement
	{
		public List<JSONStorableFloat> sliders = new List<JSONStorableFloat>();
		public List<JSONStorableBool> checks = new List<JSONStorableBool>();
		public List<JSONStorableString> headers = new List<JSONStorableString>();
		public List<UIDynamicButton> buttons = new List<UIDynamicButton>();
		
		public string targetID = "";
		public string name = "";
		public JSONStorableBool isActive;
		public bool active = false;
		public Atom atom;
		public Rigidbody body;
		public FreeControllerV3 controller;		
		public Transform controllerTransform;
		public Color normalColor = Color.blue;
		public Color activeColor = Color.yellow;
		public UIDynamicButton hostButton;
		public JSONStorableFloat timeMultiplierJSON;
		
		public virtual void Init(MVRScript script)
		{
			isActive = new JSONStorableBool(name + " Active", false);			
			script.RegisterBool(isActive);
			checks.Add(isActive);
		}
		
		public virtual void Start(MVRScript script)
		{
			//...
		}
		
		public virtual void CreateMenu(MVRScript script)
		{
			script.CreateToggle(isActive, true);
			var btn = script.CreateButton("Reset " + name, true);
			btn.buttonColor = new Color(0.5f,0.5f,0.5f);
			btn.textColor = Color.white;
			btn.button.onClick.AddListener(() => { Reset(); });
			buttons.Add(btn);
		}		
		
		public virtual void Update(float dt, float intensity)
		{
			//...
		}
		
		public virtual void OnDisable(MVRScript script)
		{
			//...
		}
		
		public virtual void SetActive(bool setActive)
		{
			if (setActive) { hostButton.buttonColor = activeColor; hostButton.textColor = Color.black; active = true; }
			else { hostButton.buttonColor = normalColor; hostButton.textColor = Color.white; active = false; }
		}
		
		public virtual void RestoreFromJson(JSONNode node)
        {
			isActive.val = node[isActive.name].AsBool;
			
            foreach (var storable in sliders)
            {
				storable.val = storable.defaultVal;
                if (node[storable.name] != null)
                {
                    storable.val = node[storable.name].AsFloat;
                }
            }
            foreach (var storable in checks)
            {
				storable.val = storable.defaultVal;
                if (node[storable.name] != null)
                {
                    storable.val = node[storable.name].AsBool;
                }
            }
            foreach (var storable in headers)
            {
				storable.val = storable.defaultVal;
                if (node[storable.name] != null)
                {
                    storable.val = node[storable.name];
                }
            }			
			if (active && !isActive.val) { SetActive(false); }
        }
		
		public virtual void Reset()
        {
			isActive.val = false;
			
            foreach (var storable in sliders)
            {
				storable.val = storable.defaultVal;
            }
            foreach (var storable in checks)
            {
				storable.val = storable.defaultVal;
            }
            foreach (var storable in headers)
            {
				storable.val = storable.defaultVal;
            }			
			if (active && !isActive.val) { SetActive(false); }
        }
	}
	
    public class AutoBody : MVRScript
    {
		public JSONStorableBool isActive;
		public JSONStorableFloat intensityJSON, timeMultiplierJSON;
		
		private List<AnimatedElement> animatedElements = new List<AnimatedElement>();
		private List<UIDynamicButton> buttons = new List<UIDynamicButton>();
		private UIDynamicButton confirmResetButton, cancelResetButton;
		private int currentIdx = -1;
		private int typeIndex = 0;
		private int typeCount = 3;
		private bool active = false;		
		private float intensity = 1f;
		private string ext = "autobody";
		private string lastDir;

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "Person")
				{
					SuperController.LogMessage("SilverAutoBody: Only works when added to a Person atom.");
					var h = CreateTextField(new JSONStorableString("header0", "Only works when added to a Person atom."),false);
					this.enabled = false;
					return;
				}
				
				//---------------------------------------------------------------------UI
				isActive = Helpers.SetupToggle(this, "Active", false, false);
				CreatePresetButtons("AutoBodyPresets", false);
				intensityJSON = Helpers.SetupSlider(this, "Overall Intensity", 1f, 0.01f, 2f, false);
				timeMultiplierJSON = Helpers.SetupSlider(this, "Random Time Multiplier", 1f, 0.01f, 2f, false);
				UIDynamicButton btn = CreateButton("Reset All", false);
				btn.buttonColor = new Color(0.5f,0.5f,0.5f);
				btn.textColor = Color.white;
				btn.button.onClick.AddListener(() => { ResetAll(); });
				btn = CreateButton("Type", false);
				btn.buttonColor = new Color(0.2f,0.2f,0.2f);
				btn.textColor = Color.white;
				btn.button.onClick.AddListener(() => { ToggleMenuType(); });
				
				//---------------------------------------------------------------------CreateCirclers
				string[] commonRigidbodies = Helpers.CommonRigidbodies();				
				for (int i = 0; i < commonRigidbodies.Length; i++)
				{
					string str = commonRigidbodies[i];
					
					Circler circler = new Circler(str, containingAtom);
					circler.Init(this);
					circler.timeMultiplierJSON = timeMultiplierJSON;
					animatedElements.Add(circler);
					
					JointDrive jointDrive = new JointDrive(str, containingAtom);
					jointDrive.Init(this);
					jointDrive.timeMultiplierJSON = timeMultiplierJSON;
					animatedElements.Add(jointDrive);
					
					RigidbodyForce rigidbodyForce = new RigidbodyForce(str, containingAtom);
					rigidbodyForce.Init(this);
					rigidbodyForce.timeMultiplierJSON = timeMultiplierJSON;
					animatedElements.Add(rigidbodyForce);
				}
				
				ToggleMenuType();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
		
		private void Start()
		{
			isActive.toggle.onValueChanged.AddListener(checkVal =>
				{
					if (checkVal) { StartAutoBody(); }
					else { StopAutoBody(); }
				});
				
			intensityJSON.slider.onValueChanged.AddListener((value) => { intensity = value; });
			
			if (isActive.val) { StartAutoBody(); }
			
			foreach (AnimatedElement ae in animatedElements)
			{
				ae.Start(this);
			}
		}
		
		private void FixedUpdate()
		{
			if (active)
			{
				float dt = Time.fixedDeltaTime;
				
				foreach (AnimatedElement ae in animatedElements)
				{
					ae.Update(dt, intensity);
				}
			}
		}
		
		private void ToggleMenuType()
		{
			foreach (UIDynamicButton button in buttons) { RemoveButton(button); }
			
			typeIndex++;
			if (typeIndex >= typeCount) { typeIndex = 0; }
			
			for (int i = 0; i < animatedElements.Count; i++)
			{
				AnimatedElement ae = animatedElements[i];
				
				if (typeIndex == 0 && ae is Circler)
				{
					UIDynamicButton btn = CreateButton(ae.targetID + " circler", false);
					btn.buttonColor = ae.active ? ae.activeColor : ae.normalColor;
					btn.textColor = ae.active ? Color.black : Color.white;
					buttons.Add(btn);
					int idx = i;
					btn.button.onClick.AddListener(() => { CreateMenu(idx); });
					ae.hostButton = btn;
				}
				else if (typeIndex == 1 && ae is JointDrive)
				{
					UIDynamicButton btn = CreateButton(ae.targetID + " joint drive", false);
					btn.buttonColor = ae.active ? ae.activeColor : ae.normalColor;
					btn.textColor = ae.active ? Color.black : Color.white;
					buttons.Add(btn);
					int idx = i;
					btn.button.onClick.AddListener(() => { CreateMenu(idx); });
					ae.hostButton = btn;
				}
				else if (typeIndex == 2 && ae is RigidbodyForce)
				{
					UIDynamicButton btn = CreateButton(ae.targetID + " rigidbody force", false);
					btn.buttonColor = ae.active ? ae.activeColor : ae.normalColor;
					btn.textColor = ae.active ? Color.black : Color.white;
					buttons.Add(btn);
					int idx = i;
					btn.button.onClick.AddListener(() => { CreateMenu(idx); });
					ae.hostButton = btn;
				}
			}
		}
		
		private void CreateMenu(int idx)
		{
			AnimatedElement ae;		
			if (currentIdx >= 0)
			{
				ae = animatedElements[currentIdx];
				foreach (JSONStorableFloat slider in ae.sliders) { RemoveSlider(slider); }
				foreach (JSONStorableBool check in ae.checks) { RemoveToggle(check); }
				foreach (UIDynamicButton button in ae.buttons) { RemoveButton(button); }
				foreach (JSONStorableString header in ae.headers) { RemoveTextField(header); }
			}
			currentIdx = idx;
			ae = animatedElements[currentIdx];
			ae.CreateMenu(this);
		}
		
		private void ResetAll()
		{
			if (currentIdx >= 0)
			{
				AnimatedElement ae = animatedElements[currentIdx];
				foreach (JSONStorableFloat slider in ae.sliders) { RemoveSlider(slider); }
				foreach (JSONStorableBool check in ae.checks) { RemoveToggle(check); }
				foreach (UIDynamicButton button in ae.buttons) { RemoveButton(button); }
				foreach (JSONStorableString header in ae.headers) { RemoveTextField(header); }
				currentIdx = -1;
			}
			
			confirmResetButton = CreateButton("CONFIRM RESET", true);
			confirmResetButton.buttonColor = new Color(0.9f,0.9f,0.9f);
			confirmResetButton.textColor = Color.black;
			confirmResetButton.button.onClick.AddListener(() => { ConfirmReset(true); });
			cancelResetButton = CreateButton("CANCEL RESET", true);
			cancelResetButton.buttonColor = new Color(0.1f,0.1f,0.1f);
			cancelResetButton.textColor = Color.white;
			cancelResetButton.button.onClick.AddListener(() => { ConfirmReset(false); });
		}
		
		private void ConfirmReset(bool confirm)
		{
			if (confirm)
			{
				StopAutoBody();
				foreach (AnimatedElement ae in animatedElements)
				{
					ae.Reset();
				}
			}
			RemoveButton(confirmResetButton);
			RemoveButton(cancelResetButton);
		}
		
		private void StartAutoBody()
		{
			active = true;
		}
		
		private void StopAutoBody()
		{
			active = false;
			foreach (AnimatedElement ae in animatedElements)
			{
				ae.OnDisable(this);
			}
		}
		
		//Credit to Physis for json save/load process
		//https://www.reddit.com/r/VAMscenes/comments/alhc6j/plugin_movement_helper/		
		private void CreatePresetButtons(string folderName, bool right)
		{		
			var btn = CreateButton("Load Preset", right);
			btn.button.onClick.AddListener(() =>
			{
				SuperController.singleton.NormalizeMediaPath("Saves\\" + folderName + "\\"); 
				SuperController.singleton.GetMediaPathDialog(LoadPreset, ext);
			});
			btn.buttonColor = Color.blue;
			btn.textColor = Color.white;

			btn = CreateButton("Save Preset", right);
			btn.button.onClick.AddListener(() =>
			{
				FileManagerSecure.CreateDirectory("Saves\\" + folderName + "\\");
				SuperController.singleton.NormalizeMediaPath("Saves\\" + folderName + "\\"); 
				SuperController.singleton.GetMediaPathDialog(SavePreset, ext);

				// Update the browser to be a Save browser
				uFileBrowser.FileBrowser browser = SuperController.singleton.mediaFileBrowserUI;
				browser.defaultPath =  "Saves\\" + folderName + "\\";
				browser.SetTextEntry(true);
				browser.fileEntryField.text = String.Format("{0}.{1}", ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString(), ext);
				browser.ActivateFileNameField();
			});
			btn.buttonColor = Color.blue;
			btn.textColor = Color.white;
		}
		
		void SavePreset(string aPath)
        {
            if (String.IsNullOrEmpty(aPath))
            {
                return;
            }
            lastDir = aPath.Substring(0, aPath.LastIndexOfAny(new char[] { '/', '\\' })) + @"\";

            if ( !aPath.ToLower().EndsWith(ext.ToLower()))
            {
                aPath += "." + ext;
            }
            JSONClass saveJson = new JSONClass();
            saveJson["savedBy"] = "ClockwiseSilver.AutoBody";            
            saveJson["AnimatedElements"] = new JSONArray();
            foreach (AnimatedElement ae in animatedElements)
            {
				JSONClass node = new JSONClass();
				ae.isActive.StoreJSON(node);
				foreach (var storable in ae.sliders)
				{
					storable.StoreJSON(node);
				}
				foreach (var storable in ae.checks)
				{
					storable.StoreJSON(node);
				}
				foreach (var storable in ae.headers)
				{
					storable.StoreJSON(node);
				}
				if (node.Count > 0)
				{
					node["id"] = ae.name;
					saveJson["AnimatedElements"].Add(node);
				}
            }
            this.SaveJSON(saveJson, aPath);
        }

        void LoadPreset(string aPath)
        {
			StopAutoBody();

            if (String.IsNullOrEmpty(aPath))
            {
                return;
            }
            lastDir = aPath.Substring(0, aPath.LastIndexOfAny(new char[] { '/', '\\' })) + @"\";
            StartCoroutine("LoadPresetRoutine", aPath);
        }
		
		private IEnumerator LoadPresetRoutine(string aPath)
		{
			bool wasActive = isActive.val;
			isActive.val = false;
			yield return null;
			
			var json = this.LoadJSON(aPath);
            foreach (JSONNode node in json["AnimatedElements"].AsArray)
            {
                string aeName = node["id"].Value;
                AnimatedElement ae = animatedElements.First(e => e.name == aeName );
				if (ae != null) { ae.RestoreFromJson(node); }
            }
			
			yield return null;
			if (wasActive) { isActive.val = true; }
		}

        private void OnDisable()
		{
			StopAutoBody();
		}
    }

}